

#ifndef _pcielm4_LIB_H_
#define _pcielm4_LIB_H_

typedef unsigned char    U8;
typedef unsigned short   U16;
typedef unsigned long    U32;
typedef	char			 I8;
typedef	short			 I16;
typedef	long			 I32;
typedef	float			 F32;
typedef	double			 F64;

//maximum of axis supported by the PCIELC4:
#define PCIELM4_MAX_AXIS_NO   (2)

#if !defined(PCIELM4_MC_EXPORTS)
//-------------------------------------------------------------------------------------------------
//pcielm4_velocity_move  - parameter 6: eDirection

#define	MOVE_DIRECTION_FORWARD	    (1)
#define	MOVE_DIRECTION_REVERSE	    (0)

//-------------------------------------------------------------------------------------------------
//pcielm4_change_p   - parameter 4: eAbsRlt
#define	RELATIVE_CMD                (0)
#define	ABSOLUTE_CMD                (1)

//-------------------------------------------------------------------------------------------------
// not in use:
#define	STATUS_OFF                 (0)
#define	STATUS_ON                  (1)

//-------------------------------------------------------------------------------------------------
// For eBitSingleAxis parameter

#define	AXIS_0                    (0x01)
#define	AXIS_1                    (0x02)
#define	AXIS_2                    (0x04)
#define	AXIS_3                    (0x08)
#define	AXIS_4                    (0x10)
#define	AXIS_5                    (0x20)


#define AXIS_X                     AXIS_0
#define AXIS_Y                     AXIS_1
#define AXIS_Z                     AXIS_2
#define AXIS_U                     AXIS_3
#define AXIS_V                     AXIS_4
#define AXIS_W                     AXIS_5

//maximum of axis supported by the PCIELC4:
#define	MAX_AXIS_QTY                (PCIELM4_MAX_AXIS_NO)
#define MAX_AXIS_NO					(PCIELM4_MAX_AXIS_NO)


//-------------------------------------------------------------------------------------------------
// pcielm4_lines_move        parameter 3: eAccDecMode
// pcielm4_abs_lines_move    parameter 3: eAccDecMode
#define	ACC_DEC_T_CURVE    (0x6565)
#define	ACC_DEC_S_CURVE    (0x6666)


//-------------------------------------------------------------------------------------------------
//pcielm4_set_softlimit           parameter 4: eFeedbackSrc
//pcielm4_set_pos_feedback_src    parameter 3: eSpeedfeedbackMode
//pcielm4_set_speed_feedback_src  parameter 3: eSpeedfeedbackMode
#define	FEEDBACK_SRC_DDA_ENC         (0)	// Theoretical Command with one cycle delay
#define	FEEDBACK_SRC_ENC             (1)	// Real Closed Loop form Encoder
#define	FEEDBACK_SRC_DDA             (2)	// Internal Loop back from DDA
#define	FEEDBACK_SRC_ADC             (3)	// Real Closed Loop form ADC, for speed feedback only
#define	FEEDBACK_SRC_DAC             (4)	// Internal Loop back from DAC, for speed feedback only


//-------------------------------------------------------------------------------------------------
//pcielm4_set_axis_do
//pcielm4_set_servo_on
//pcielm4_set_erc
//pcielm4_set_alarm_reset
//pcielm4_get_general_di
//pcielm4_set_general_do
//pcielm4_get_general_do
//pcielm4_get_frnet_di
//pcielm4_get_servo_on_status
//pcielm4_get_servo_erc_status
//pcielm4_get_servo_almrst_status
#define	OFF					        (0x0)
#define	ON					        (0x1)

//-------------------------------------------------------------------------------------------------
//pcielm4_set_axis_do     parameter 3: eDOChannel
#define	AXIS_DO_SVON_CH             (0x0)
#define	AXIS_DO_ERC_CH              (0x1)
#define	AXIS_DO_ALMRST_CH           (0x2)
#define	AXIS_DO_RESERVED_CH         (0x3)

//-------------------------------------------------------------------------------------------------
//pcielm4_set_axis_di_filter   parameter 4: eAxisDiChannel
#define	AXIS_DI_EA_CH               (11)
#define	AXIS_DI_EB_CH               (10)
#define	AXIS_DI_EZ_CH               (9)		
#define	AXIS_DI_IDX_CH              (8)
#define	AXIS_DI_LTC_CH              (7)
#define	AXIS_DI_RDY_CH              (6)
#define	AXIS_DI_INP_CH              (5)		//servo in position
#define	AXIS_DI_ALM_CH              (4)		//servo alarm
#define	AXIS_DI_SLD_CH              (3)		//Near Home 
#define	AXIS_DI_ORG_CH              (2)		//Home 
#define	AXIS_DI_MEL_CH              (1)		//negative hardware limit
#define	AXIS_DI_PEL_CH              (0)		//positive hardware limit

//-------------------------------------------------------------------------------------------------
//pcielm4_set_general_do          parameter 2: eDoChannelNo
//pcielm4_get_general_do          parameter 2: eDoChannelNo
#define	DO_GDO2_CH                  (2) 
#define	DO_GDO1_CH                  (1) 
#define	DO_GDO0_CH                  (0) 

//-------------------------------------------------------------------------------------------------
//Each card  has a couple of axis independent DI channels:
//pcielm4_set_general_di_pol      parameter 2: eDiChannelNo
//pcielm4_get_general_di_pol      parameter 2: eDiChannelNo
//pcielm4_get_general_di          parameter 2: eDiChannelNo

#define	DI_EMG_CH                   (15)
#define	DI_MPG_EMG_CH               (14)
#define	DI_MPG_A_CH                 (13)
#define	DI_MPG_B_CH                 (12)
#define	DI_GDI11_CH                 (11)
#define	DI_GDI10_CH                 (10)
#define	DI_GDI9_CH                  (9)
#define	DI_GDI8_CH                  (8)
#define	DI_GDI7_CH                  (7)
#define	DI_GDI6_CH                  (6)
#define	DI_GDI5_CH                  (5)
#define	DI_GDI4_CH                  (4)
#define	DI_GDI3_CH                  (3)
#define	DI_GDI2_CH                  (2)
#define	DI_GDI1_CH                  (1)
#define	DI_GDI0_CH                  (0)	


 
//-------------------------------------------------------------------------------------------------
// Filter Configuration in pcielm4_set_axis_io_filter or pcielm4_set_di_filter
//
//		For EA/EB/EZ and LTC
//			FILTER_DISABLE_FEATURE
//			FILTER_DELAY_0p12us
//			FILTER_DELAY_0p48us
//			FILTER_DELAY_1p92us
//		For PEL/MEL/ORG/SLD, MPG_EMG/MPG_A/MPG_B and EMG
//			FILTER_DISABLE_FEATURE
//			FILTER_DELAY_3p84us
//			FILTER_DELAY_30p72us
//			FILTER_DELAY_245p76us
//		For ALM/INP/RDY and DI0~DI11
//			FILTER_DISABLE_FEATURE
//			FILTER_DELAY_3p84us

//pcielm4_set_axis_di_filter     parameter 3: eFilterEnable
//pcielm4_set_di_filter          parameter 2: eFilterEnable
#define	FILTER_DELAY_DISABLE        (0)     //disable the use of filter delay
#define	FILTER_DELAY_0p12us			(1)
#define	FILTER_DELAY_0p48us			(2)
#define	FILTER_DELAY_1p92us			(3)
#define	FILTER_DELAY_3p84us			(4)
#define	FILTER_DELAY_30p72us		(5)
#define	FILTER_DELAY_245p76us		(6)



//-------------------------------------------------------------------------------------------------
//for done status in the pcielm4_motion_done() 
#define MOTION_DONE					1
#define MOTION_NOT_DONE				0
//-------------------------------------------------------------------------------------------------
//pcielm4_set_limit          parameter 4: eStopMode
//pcielm4_set_softlimit      parameter 3: eStopMode
//pcielm4_stop_move          parameter 3: eStopMode
#define	STOP_NONE                  (0)
#define	STOP_SLOWDOWN              (1)
#define	STOP_SUDDEN                (2)


//-------------------------------------------------------------------------------------------------
//pcielm4_set_limit
//pcielm4_set_home_cfg
//pcielm4_set_alarm
//pcielm4_set_inp
//pcielm4_set_ready
//pcielm4_set_compare_trig_cfg
//pcielm4_set_general_di_pol
//pcielm4_get_general_di_pol
//pcielm4_set_latch_cfg
#define	LOGIC_ACTIVE_LOW          (0x0000)
#define	LOGIC_ACTIVE_HIGH         (0x0001)

//-------------------------------------------------------------------------------------------------
// Auto-Homing Steps Configuration 
//pcielm4_home_start
#define AUTO_HOME_STEP1_DISABLE				0x00  
#define AUTO_HOME_STEP1_ENABLE				0x01  // Enable near Home search

#define AUTO_HOME_STEP2_DISABLE				0x00
#define AUTO_HOME_STEP2_ENABLE				0x04  // Enable Home search

#define AUTO_HOME_STEP3_DISABLE				0x00
#define AUTO_HOME_STEP3_ENABLE				0x10  // Enable z-index search

#define AUTO_HOME_STEP4_DISABLE				0x00
#define AUTO_HOME_STEP4_ENABLE				0x40  // Enable offset move



#define	AUTO_HOME_FORWARD					(0x0001) 
#define	AUTO_HOME_REVERSE					(0x0000)


//-------------------------------------------------------------------------------------------------
// Compare & Trigger Configuration
//pcielm4_set_compare_trig_cfg
#define	CMPTRIG_FORWARD_MOVE				MOVE_DIRECTION_FORWARD
#define	CMPTRIG_REVERSE_MOVE				MOVE_DIRECTION_REVERSE


#define	TRIG_PULSE_WIDTH_ACTUAL_PULSE			(0x0000)
#define	TRIG_PULSE_WIDTH_160nsTo320ns			(0x0001)
#define	TRIG_PULSE_WIDTH_320nsTo640ns			(0x0002)
#define	TRIG_PULSE_WIDTH_640nsTo1p28us			(0x0003)
#define	TRIG_PULSE_WIDTH_1p28usTo2p56us			(0x0004)
#define	TRIG_PULSE_WIDTH_2p56usTo5p12us			(0x0005)
#define	TRIG_PULSE_WIDTH_5p12usTo10p24us		(0x0006)
#define	TRIG_PULSE_WIDTH_10p24usTo20p48us		(0x0007)
#define	TRIG_PULSE_WIDTH_20p48usTo40p96us		(0x0008)
#define	TRIG_PULSE_WIDTH_40p96usTo81p92us		(0x0009)
#define	TRIG_PULSE_WIDTH_81p92usTo163p84us		(0x000A)
#define	TRIG_PULSE_WIDTH_163p84usTo327p68us		(0x000B)
#define	TRIG_PULSE_WIDTH_327p68usTo655p36us		(0x000C)
#define	TRIG_PULSE_WIDTH_655p36usTo1p31072ms	(0x000D)
#define	TRIG_PULSE_WIDTH_1p31072msTo2p62144ms	(0x000E)
#define	TRIG_PULSE_WIDTH_2p62144msTo5p24288ms	(0x000F)


//-------------------------------------------------------------------------------------------------
//pcielm4_set_alarm
//pcielm4_set_inp
//pcielm4_set_ready
//pcielm4_set_axis_di_filter
//pcielm4_set_di_filter
//pcielm4_set_compare_trig_cfg
//pcielm4_set_latch_cfg
#define	DISABLE                                (0x0)
#define	ENABLE                                 (0x1)


//-------------------------------------------------------------------------------------------------
// Output Pulse Configuration
//pcielm4_set_pls_cfg
//ePulseMode:
#define	PULSE_MODE_PULSE_DIRECTION	           (0x0)
#define	PULSE_MODE_CW_CCW			           (0x1)
#define	PULSE_MODE_AB_DIVID_4		           (0x2)  	// Quadrature pulse input

//ePulseLogic:
#define	PULSE_LOGIC_ACTIVE_HIGH                (0x0)
#define	PULSE_LOGIC_ACTIVE_LOW                 (0x1)

//eDirectionLogic:

#define	PULSE_FORWARD_ACTIVE_HIGH              (0x0)
#define	PULSE_FORWARD_ACTIVE_LOW               (0x1)


#define PULSE_FORWARD_CW                       (0x0)
#define PULSE_FORWARD_CCW                      (0x1)

//-------------------------------------------------------------------------------------------------
//Encoder Configuration
//pcielm4_set_enc_cfg
//eEncoderMode:
#define	ENCODER_MODE_CW_CCW			           (0x1)  // Up/Down pulse input
#define	ENCODER_MODE_AB_DIVID_4		           (0x2)  // Quadrature pulse input
#define	ENCODER_MODE_AB_DIVID_2		           (0x3)
#define	ENCODER_MODE_AB				           (0x4)


//-------------------------------------------------------------------------------------------------
//pcielm4_load_power_on_cfg
#define	DEFAULT_POWER_ON_SETTING               (0)   // factory default settings
#define	SAVED_POWER_ON_SETTING                 (1)   //loads configuration settings stored in the FRAM

//-------------------------------------------------------------------------------------------------
// Interporation Arc move
//pcielm4_t_arc2_move                    parameter: bArcDirection
//pcielm4_abs_t_arc2_move                parameter: bArcDirection
// 
#define	ARC_DIR_CW                            (0x0000)    //cw      //INTERP_ARC_DIRECTION_FORWARD
#define	ARC_DIR_CCW                           (0x0001)     //ccw     //INTERP_ARC_DIRECTION_REVERSE


//-------------------------------------------------------------------------------------------------
// Enable / Cancle the configuration of continue-interplation
//pcielm4_set_conti_interp_cfg          parameter: eCfgEnable
#define	CANCEL_INTERP_CONFIG                 (0x0000)
#define	START_INTERP_CONFIG                  (0x0001)



//-------------------------------------------------------------------------------------------------
// Returned status in pcielm4_motion_done();
#define DRIVE_STILL_DRIVING         		    0x0000
#define DRIVE_FINISH_OUTPUT_FIXED_PULSE		    0x0001
#define DRIVE_FINISH_AUTO_HOME					0x0002
#define DRIVE_FINISH_STOP_COMMAND				0x0004
#define DRIVE_FINISH_WAIT_FOR_INPOS             0x0008  // wait for servo driver in-position signal

#define DRIVE_FINISH_SW_LIMIT_POSITIVE			0x0010  //software limit positive
#define DRIVE_FINISH_SW_LIMIT_NEGATIVE			0x0020  //software limit negative
#define DRIVE_FINISH_LIMIT_POSITIVE			    0x0040  //hardware limit positive
#define DRIVE_FINISH_LIMIT_NEGATIVE			    0x0080  //hardware limit negative
#define DRIVE_FINISH_ALARM						0x0100
#define DRIVE_FINISH_EMG						0x0200


//-------------------------------------------------------------------------------------------------
// pcielm4_get_error_status();
#define DRIVE_ERROR_STATUS_SOFT_LMT_P				0x0001 //software positive limit error
#define DRIVE_ERROR_STATUS_SOFT_LMT_M				0x0002 //software negative limit error
#define DRIVE_ERROR_STATUS_LMTP						0x0004 //hardware positive limit error
#define DRIVE_ERROR_STATUS_LMTM						0x0008 //hardware negative limit error
#define DRIVE_ERROR_STATUS_ALARM					0x0010
#define DRIVE_ERROR_STATUS_EMG						0x0020
#define DRIVE_ERROR_STATUS_HOME						0x0080


//-------------------------------------------------------------------------------------------------
// DI Status Configuration

#define DI_STATUS_ALARM								0x0010
#define DI_STATUS_HOME								0x0020
#define DI_STATUS_NEARHOME							0x0040
#define DI_STATUS_INP								0x0080
#define DI_STATUS_INDEX								0x0100

#define DI_STATUS_ACTIVE_DRIVING					0x0001
#define DI_STATUS_ACTIVE_LMTP						0x0002
#define DI_STATUS_ACTIVE_LMTM						0x0004
#define DI_STATUS_ACTIVE_EMG						0x0008
#define DI_STATUS_ACTIVE_ALARM						DI_STATUS_ALARM
#define DI_STATUS_ACTIVE_HOME						DI_STATUS_HOME
#define DI_STATUS_ACTIVE_NEARHOME					DI_STATUS_NEARHOME
#define DI_STATUS_ACTIVE_INP						DI_STATUS_INP
#define DI_STATUS_ACTIVE_INDEX						DI_STATUS_INDEX
#define DI_STATUS_ACTIVE_RDY						0x0200
#define DI_STATUS_ACTIVE_LATCH						0x0400



//-------------------------------------------------------------------------------------------------
// Continuous interpolation mode:
//pcielm4_t_line2_move                   parameter: eInterpMode
//pcielm4_abs_t_line2_move
//pcielm4_s_line2_move
//pcielm4_abs_s_line2_move
//pcielm4_t_line3_move
//pcielm4_abs_t_line3_move
//pcielm4_s_line3_move
//pcielm4_abs_s_line3_move
//pcielm4_t_arc2_move
//pcielm4_abs_t_arc2_move
//pcielm4_abs_t_arc2_move
//pcielm4_t_helical_move
//pcielm4_abs_t_helical_move
//pcielm4_lines_move
//pcielm4_abs_lines_move
#define	GENERAL_INTERP_CONFIG               (0x0)  //start stop mode
#define	CONTI_INTERP_CONFIG                 (0x1)
#define	CONTI_FIFO_BUFFER_MODE              (0x2)  //Currently not supported. Commands will first be stored in an internal FIFO buffer of this DLL. To execute the FIFO buffer commands call "pcielm4_conti_interp_start_all()"

//-------------------------------------------------------------------------------------------------
// for Control mode in pcielm4_set_ctrl_mode

#define	CTL_MODE_PULSE                      (1)   //factory default setting
//For customer request 
#define	CTL_MODE_VIRTUAL_PULSE              (11) 

//-------------------------------------------------------------------------------------------------
//  Buffer parameter:
//-------------------------------------------------------------------------------------------------

//eBUFFER_CONTI:
#define	BUFFER_CONTI_NOT_FINISH             (0)
#define	BUFFER_CONTI_FINISH                 (1)


//eBUFFER_LOOP:
#define	BUFFER_LOOP_DISABLE                 (0)
#define	BUFFER_LOOP_ENABLE                  (1)

//eBUFFER_START:
#define	BUFFER_RESUME_FROM_PAUSE            (0)
#define	BUFFER_RESTART_ALL                  (1)


//pcielm4_buffer_initial
//pcielm4_buffer_reinit
//pcielm4_buffer_release
//pcielm4_buffer_continue
//pcielm4_buffer_start
#define	BUFFER_TYPE_NULL                    (0)
#define	BUFFER_TYPE_FPGA                    (1)
#define	BUFFER_TYPE_FLASH                   (2)
#define	BUFFER_TYPE_DDAC                    (3)
#define	BUFFER_TYPE_CMP                     (4)
#define	BUFFER_TYPE_LTC                     (5)
#define	BUFFER_TYPE_MOVE                    (6)
#define	BUFFER_TYPE_MAX_MOVE_NO             (50)



//-------------------------------------------------------------------------------------------------

//============================================================================
//====================== PCIELM4_MC DLL Function return values===================
//============================================================================
#define	PCIELM4_SUCCESS                    (0)
#define	PCIELM4_ERROR                      (1)
#define	ERR_CARD_NOT_FOUND                 (2)
#define	ERR_INVALID_SLOT_NO                (3)
#define	ERR_INVALID_PARA                   (4)
#define	ERR_INVALID_DRIVING_SPEED          (5)
#define	ERR_INVALID_START_SPEED            (6)
#define ERR_INVALID_END_SPEED              (7)
#define	ERR_INVALID_MPG_SPEED              (8)
#define	ERR_INVALID_MOVE_DIRECTION         (9)
#define	ERR_AXES_NOT_MATCH                 (10)

#define	ERR_GROUP_INVALID_DIMENSION        (11)
#define	ERR_INVALID_GROUP_AXES             (12)
#define	ERR_GROUP_NOT_CONFIGURED           (13)
#define	ERR_GROUP_OUT_OF_RANGE             (14)       //invalid Group input parameter
#define	ERR_INCORRECT_GROUP_ASSIGNED       (15)
#define	ERR_GROUP_SAME_AXIS                (16)
#define	ERR_GROUP_RELEASE_REQUIRED         (17)

#define	ERR_MOTION_NOT_FINISHED            (18)
#define	ERR_CARD_NOT_REGISTERED            (19)
#define	ERR_INVALID_AXIS                   (20)
#define	ERR_AXIS_ERROR                     (21)              //motion stop due to an axis error (e.g. hardware limit, alarm). Call GetAxisError() to determine the cause of the error
#define	ERR_CMD_NO_CLOSED_LOOP_SUPPORT     (22)
#define	ERR_INTERPOL_NOT_CONFIGURED        (23)
#define	ERR_INVALID_HELICAL_MODE           (24)
#define	ERR_INVALID_FRNET_SPEED_SETTING    (25)
#define	ERR_INVALID_FRNET_SA_GROUP_ADDR    (26)
#define	ERR_INVALID_FRNET_RA_GROUP_ADDR    (27)
#define	ERR_INVALID_MPG_GAIN               (28)
#define	ERR_MPG_NOT_CONFIGURED             (29)
#define	ERR_MOTION_IS_COMPLETED            (30)
	
#define	ERR_CMD_NOT_FOR_CONTI_INTERPOL     (31)
#define	ERR_MODE_NOT_FOR_CONTI_INTERPOL    (32)
#define	ERR_DPRAM_RTC_BLOCKS_AVAILABLE     (33)
#define	ERR_DPRAM_RTC_BUFFER_FULL          (34)   // All 24 RTC DPRAM buffer are filled with commands and still has to be processed
#define	EMG_ACVTIVATED                     (35)	  //emergency has been activated

#define	ERR_CONFLICT_CONTI_INTERP_CONFIG   (36)
#define	ERR_CONTI_INTERP_INVALID_CONFIG    (37)
#define	ERR_CONTI_INTERP_INTERRUPTED       (38)
#define	ERR_CONTI_INTERP_INVALID_START     (39)
#define	ERR_CONTI_INTERP_FIFO_EMPTY        (40)
#define	ERR_NOT_IN_CONTI_INTERP_MODE       (41)
#define	ERR_CONTI_INTERP_START             (42)
#define	ERR_INVALID_RING_COUNTER           (43)
#define	ERR_ISR_IS_USED_FOR_CMP_TRIG       (44)

#define	ERR_RTC_WAIT_ABORTED               (45)
#define	ERR_RTC_TIMEOUT                    (46)
#define	ERR_DSP_RESET                      (47)
#define	ERR_DDA_IS_BUSY                    (48)
#define ERR_EXCEED_MAX_POSTION             (49)
#define ERR_NOT_SUPPORTED_BY_VRING_MODE    (50)
#define ERR_DLL_DO_NOT_SUPPORT_MULTI_APP   (51)
#define ERR_DRIVER_NOT_OPEN                (52)
#define ERR_NO_BOARD                       (53)
#define ERR_API_NOT_SUPPORTED              (54)

#define ERR_RESERVED_SPACE                 (70)
//Buffer APIs return:
#define	ERR_BUFFER_UNSUPPORTED_TYPE        (71)
#define	ERR_BUFFER_INVALID_DATA            (72)
#define	ERR_BUFFER_INVALID_CONTI_SIZE      (73)
#define	ERR_BUFFER_INVALID_CONTI_FINISH    (74)
#define	ERR_BUFFER_START                   (75)
#define	ERR_BUFFER_INVALID_LOOP            (76)
#define	ERR_BUFFER_INVALID_RESTART         (77)
    

	//DSP return code:
#define	ERR_DSP_SYSTEM_ERROR             (-32767)
#define	ERR_DSP_NOT_IMPLEMENT            (-32765)
#define	ERR_DSP_MOTIONPATH_FULL          (-1001)
#define	ERR_DSP_REMAIN_STILL             (-1002)
#define	ERR_DSP_DIMENSION_OUT_RANGE      (-1003)
#define	ERR_DSP_SPEED_VALUE              (-1004)
#define	ERR_DSP_SMALLSHAPE_ERROR         (-1005)
#define	ERR_DSP_SAWSHAPE_ERROR           (-1006)
#define	ERR_DSP_AXIS_INUSE               (-1007)
#define	ERR_DSP_NORMALVECTOR             (-1008)
#define	ERR_DSP_CIRCLE_RADIUS            (-1009)
#define ERR_DSP_MECH_PARA_VALUE          (-1011)

#define	ERR_DSP_MOTIONPATH_ALREADY_FREE  (-1100)
#define	ERR_DSP_EXCEED_SPEED_LIMIT       (-1201)
#define ERR_DSP_EXCEED_VRING_LIMIT       (-1202)
#define ERR_DSP_EXCEED_MAX_POSTION       (-1203)

#define	ERR_DSP_NO_COMMAND               (-2001)
#define	ERR_DSP_CRC_ERROR                (-2002)
#define	ERR_DSP_UNKNOWN_COMMAND          (-2003)
#define	ERR_DSP_MULTIBLOCK_CMD           (-2004)
#define	ERR_DSP_ACC_TYPE                 (-2005)
#define	ERR_DSP_DEC_TYPE                 (-2006)
#define	ERR_DSP_CMD_NOT_ALLOWED          (-2007)

#define	ERR_DSP_INHIBIT_BY_EMG           (-2011)
#define	ERR_DSP_INHIBIT_BY_MPG_EMG       (-2012)
#define	ERR_DSP_INHIBIT_BY_PEL           (-2013)
#define	ERR_DSP_INHIBIT_BY_MEL           (-2014)
#define	ERR_DSP_INHIBIT_BY_ALM           (-2015)
#define	ERR_DSP_INHIBIT_BY_RDY           (-2016)

#define	ERR_DSP_GROUP_OUT_RANGE          (-2101)
#define	ERR_DSP_GROUP_ALREADY_FREE       (-2102)
#define	ERR_DSP_GROUP_ASSIGNED           (-2103)
#define	ERR_DSP_GROUP_INUSE              (-2104)
#define	ERR_DSP_GROUP_NOT_INUSE          (-2105)
#define	ERR_DSP_AXIS_ASSIGNED            (-2106)

#define	ERR_DSP_CONTROL_MODE             (-2201)

//Buffer APIs return:
#define	ERR_DSP_BUFFER_FULL              (-2301)
#define	ERR_DSP_BUFFER_INFO              (-2302)
#define	ERR_DSP_BUFFER_TYPE              (-2303)
#define	ERR_DSP_BUFFER_SIZE              (-2304)
#define	ERR_DSP_BUFFER_INUSE             (-2305)
#define	ERR_DSP_BUFFER_NOT_READY         (-2306)

#define	ERR_DSP_STOP_BY_P_CHANGE         (-2401)
#define	ERR_DSP_CLEAR_STOP               (-2402)

#define	ERR_DSP_STOP_BY_GINP             (-2410)
#define	ERR_DSP_STOP_BY_AXIS_IO          (-2430)

#define	ERR_DSP_NO_AVAILABLE_MACRO       (-2501)
#define	ERR_DSP_MACRO_INUSE              (-2502)
#define	ERR_DSP_MACRO_EMPTY              (-2503)

#define	ERR_DSP_HOMING_IN_PROGRESS       (-2601)

#define	ERR_DSP_OUTPUT_SATURATION        (-3001)
#define	ERR_DSP_ERR_MSG_BUF_EMPTY        (-3011)
#define	ERR_DSP_ERR_MSG_BUF_OVERFLOW     (-3012)
#define ERR_DSP_ERR_TEST_FRAM            (-4001)
#define ERR_DSP_ERR_TEST_DPRAM           (-4002)

#endif //#if !defined(PCIELM4_MC_EXPORTS)


 #if defined(PCIELM4_MC_EXPORTS)
	#ifdef __cplusplus
		#define EXPORTS extern "C" //__declspec(dllexport)
		#define DEFAULT_VALUE(value) = value
	#else
		#define EXPORTS __declspec(dllexport)
		#define DEFAULT_VALUE(value)
	#endif
 #else
	#ifdef __cplusplus
		#define EXPORTS extern "C" __declspec(dllimport)
		#define DEFAULT_VALUE(value) = value
	#else
		#define EXPORTS __declspec(dllimport)
		#define DEFAULT_VALUE(value)
	#endif
#endif
	
//#if !defined(_WIN32_WCE)
//	#define CALLCONV __stdcall// calling convention
//#endif

//============================================================================
//====================== PCIELM4_MC DLL Release Functions===================
//============================================================================
EXPORTS	I32 CALLBACK pcielm4_open_driver(void);   //Only valid for PCIe card
EXPORTS	I32 CALLBACK pcielm4_close_driver(void);  //Only valid for PCIe card

EXPORTS	I32 CALLBACK pcielm4_registration( U16 wBoardNo );
EXPORTS	I32 CALLBACK pcielm4_check_card_in_slot(U16 wBoardNo);
EXPORTS	I32 CALLBACK pcielm4_get_card_version( U16 wBoardNo, U16* pwPCBVersion,U16* pwPLDVersion);
EXPORTS	I32 CALLBACK pcielm4_get_fpga_version( U16 wBoardNo, U16* pwFpgaVers );
EXPORTS	I32 CALLBACK pcielm4_get_dsp_firmware_version( U16 wBoardNo, U16* pwDspFirmwareVers );
EXPORTS	I32 CALLBACK pcielm4_get_dll_version( U16* pwDllVers );

EXPORTS	I32 CALLBACK pcielm4_load_power_on_cfg(U16 wBoardNo, I32 eCfgSource );
EXPORTS	I32 CALLBACK pcielm4_save_power_on_cfg(U16 wBoardNo);

EXPORTS	I32 CALLBACK pcielm4_get_motion_done( U16 wBoardNo, U8 bSingleAxis, U8* pbDone, U16* pwStopStatus	);
EXPORTS	I32 CALLBACK pcielm4_get_cmdcounter(U16 wBoardNo, U8 bSingleAxis, I32* plLogicPosCount );
EXPORTS	I32 CALLBACK pcielm4_get_enccounter(U16 wBoardNo, U8 bSingleAxis, I32* plEncoderPosCount );
EXPORTS	I32 CALLBACK pcielm4_get_position( U16 wBoardNo, U8 bSingleAxis, I32* plPos );
EXPORTS	I32 CALLBACK pcielm4_get_max_speed(U16 wBoardNo, U8 bSingleAxis, U32* pdwMaxSpeed );
EXPORTS	I32 CALLBACK pcielm4_get_speed(U16 wBoardNo, U8 bSingleAxis, I32* plSpeed );
EXPORTS	I32 CALLBACK pcielm4_get_acc( U16 wBoardNo, U8 bSingleAxis, I32* plAcc	);


//==================================================================
//------------ RTC Commands
//==================================================================

EXPORTS	I32 CALLBACK pcielm4_set_pls_cfg(U16 wBoardNo, U8 bSingleAxis, U16 wPulseMode,  U8 bPulseLogic, U8 bDirectionLogic );
EXPORTS	I32 CALLBACK pcielm4_set_enc_cfg(U16 wBoardNo, U8 bSingleAxis, U16 wEncoderMode );
EXPORTS	I32 CALLBACK pcielm4_set_limit(U16 wBoardNo, U8 bSingleAxis,  U8 bLimitLogic, U8 bStopMode );
EXPORTS	I32 CALLBACK pcielm4_set_softlimit( U16 wBoardNo, U8 bSingleAxis, U8 bStopMode,	U8 bRefSource,	I32 lLimitPositive, I32 lLimitNegative);
EXPORTS	I32 CALLBACK pcielm4_set_softlimit_disable( U16 wBoardNo, U8 bSingleAxis );

EXPORTS	I32 CALLBACK pcielm4_set_max_speed(U16 wBoardNo, U8 bSingleAxis,  U32 dwMaxSpeed );
EXPORTS	I32 CALLBACK pcielm4_set_home_cfg(U16 wBoardNo, U8 bSingleAxis, U8 bHomeLogic, U8 bNHomeLogic,	  
									U8 bIndexLogic, U8 bHomeSteps,	I32 lStep4Offset);
EXPORTS	I32 CALLBACK pcielm4_home_start(U16 wBoardNo, U8 bSingleAxis, U32 dwStartSpeed, U32 dwAcceleration, U32 dwDeceleration,		
	                             U32 dwNHomeSearchSpeed, U32 dwHomeSearchSpeed,	U8 bHomingDirection );
//Motion DO Setting:
EXPORTS	I32 CALLBACK pcielm4_set_axis_do( U16 wBoardNo, U8 bSingleAxis, U8 bDOChannel, U8 bOnOff );
EXPORTS	I32 CALLBACK pcielm4_set_servo_on( U16 wBoardNo, U8 bSingleAxis,  U8 bServoOnOff);
EXPORTS	I32 CALLBACK pcielm4_set_erc( U16 wBoardNo, U8 bSingleAxis,  U8 bErcOnOff );
EXPORTS	I32 CALLBACK pcielm4_set_alarm_reset( U16 wBoardNo, U8 bSingleAxis, U8 bAlmRstOnOff );
//Motion DI Setting:
EXPORTS	I32 CALLBACK pcielm4_set_alarm( U16 wBoardNo,  U8 bSingleAxis, U8 bEnableDisable,  U8 bTrigLevel );
EXPORTS	I32 CALLBACK pcielm4_set_inp( U16 wBoardNo, U8 bSingleAxis, U8 bEnableDisable, U8 bINPLogic );
EXPORTS	I32 CALLBACK pcielm4_set_ready(U16 wBoardNo, U8 bSingleAxis, U8 bEnableDisable, U8 bRdyLogic );
EXPORTS	I32 CALLBACK pcielm4_set_axis_di_filter( U16 wBoardNo, U8 bSingleAxis, U8 bFilterEnable, U16 wAxisDiChannel,	U16 wDelayTime );
//EXPORTS	I32 CALLBACK pcielm4_set_di_filter( U16 wBoardNo, U8 bFilterEnable, U16 wGeneralDiChannel, U16 wDelayTime );

EXPORTS	I32 CALLBACK pcielm4_set_cmdcounter(U16 wBoardNo, U8 bSingleAxis, I32 lLogicPos	);
EXPORTS	I32 CALLBACK pcielm4_set_enccounter(U16 wBoardNo, U8 bSingleAxis, I32 lEncPos );
EXPORTS	I32 CALLBACK pcielm4_set_position(U16 wBoardNo, U8 bSingleAxis, I32 lLogicPos );

EXPORTS	I32 CALLBACK pcielm4_set_vring_counter( U16 wBoardNo, U8 bSingleAxis, U32 dwRingValue	);
EXPORTS	I32 CALLBACK pcielm4_get_vring_counter( U16 wBoardNo, U8 bSingleAxis,  U32 *pdwRingValue);
EXPORTS	I32 CALLBACK pcielm4_disable_vring_counter( U16 wBoardNo, U8 bSingleAxis );

EXPORTS	I32 CALLBACK pcielm4_set_compare_trig_cfg(U16 wBoardNo,  U8 bSingleAxis, U8 bCmpTrigEnable, U8 bOutputLogic,	
                                            U16 wPulseWidth, U8 bMoveDirection, U8 bCmpIncEnable,	
	                                        U16 wConstPitch, I32 lCmpData );

EXPORTS	I32 CALLBACK pcielm4_set_conti_interp_cfg(U16 wBoardNo, U8 bCfgEnable, U16 wGroupIndex0 );

EXPORTS	I32 CALLBACK pcielm4_velocity_move(U16 wBoardNo, U8 bSingleAxis,	
                                            U32 dwStartSpeed,	U32 dwDriveSpeed, U32 dwAcceleration, U8 bDirection );

//--------------------------------------------------------------------------------------------------------------------------------
EXPORTS	I32 CALLBACK pcielm4_const_move(U16 wBoardNo, U8 bSingleAxis,	U32 dwDriveSpeed,	I32  lFixedPulse);
EXPORTS	I32 CALLBACK pcielm4_abs_const_move( U16 wBoardNo, U8 bSingleAxis, U32 dwDriveSpeed, I32  lFixedPulse );
//--------------------------------------------------------------------------------------------------------------------------------
EXPORTS	I32 CALLBACK pcielm4_t_move( U16 wBoardNo, U8 bSingleAxis, 
								      U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, 	
								      U32 dwAcceleration, U32 dwDeceleration, I32 lFixedPulse);
EXPORTS	I32 CALLBACK pcielm4_abs_t_move( U16 wBoardNo, U8 bSingleAxis, 
								          U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, 	
								          U32 dwAcceleration, U32 dwDeceleration, I32 lFixedPulse);
//--------------------------------------------------------------------------------------------------------------------------------
EXPORTS	I32 CALLBACK pcielm4_s_move( U16 wBoardNo, U8 bSingleAxis, 
								      U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, 	
								      U32 dwAcceleration, U32 dwDeceleration, I32 lFixedPulse );
EXPORTS	I32 CALLBACK pcielm4_abs_s_move( U16 wBoardNo, U8 bSingleAxis, 
								          U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed,	
								          U32 dwAcceleration, U32 dwDeceleration, I32 lFixedPulse );
//--------------------------------------------------------------------------------------------------------------------------------
EXPORTS	I32 CALLBACK pcielm4_t_line2_move(U16 wBoardNo, U8 bMainAxis, U8 bSlaveAxis,	
	                                       U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, U32 dwAcceleration,	U32 dwDeceleration,		
	                                       I32 lMainAxisRelDist, I32 lSlaveAxisRelDist, 
                                           U16 wInterpMode	);

EXPORTS	I32 CALLBACK pcielm4_abs_t_line2_move(U16 wBoardNo, U8 bMainAxis, U8 bSlaveAxis,	
	                                           U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, U32 dwAcceleration,	U32 dwDeceleration,		
	                                           I32 lMainAxisFinishPoint, I32 lSlaveAxisFinishPoint,	
	                                           U16 wInterpMode	);
//--------------------------------------------------------------------------------------------------------------------------------
EXPORTS	I32 CALLBACK pcielm4_s_line2_move(U16 wBoardNo, U8 bMainAxis, U8 bSlaveAxis,	
	                                       U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, U32 dwAcceleration,	U32 dwDeceleration,		
	                                       I32 lMainAxisRelDist, I32 lSlaveAxisRelDist,	
	                                       U16 wInterpMode);

EXPORTS	I32 CALLBACK pcielm4_abs_s_line2_move(U16 wBoardNo, U8 bMainAxis, U8 bSlaveAxis,	
	                                           U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, U32 dwAcceleration,	U32 dwDeceleration,		
	                                           I32 lMainAxisFinishPoint, I32 lSlaveAxisFinishPoint,	
	                                           U16 wInterpMode );

//--------------------------------------------------------------------------------------------------------------------------------
EXPORTS	I32 CALLBACK pcielm4_t_arc2_move( U16 wBoardNo, U8 bMainAxis, U8 bSlaveAxis,			
	                                      U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, U32 dwAcceleration, U32 dwDeceleration,  U8 bArcDirection,			    
	                                      I32 lMainAxisCenterPoint,	I32 lSlaveAxisCenterPoint, I32 lMainAxisFinishPoint, I32 lSlaveAxisFinishPoint,	        
	                                      U16 wInterpMode	);


EXPORTS	I32 CALLBACK pcielm4_abs_t_arc2_move( U16 wBoardNo, U8 bMainAxis, U8 bSlaveAxis,			
	                                           U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, U32 dwAcceleration, U32 dwDeceleration, U8 bArcDirection,			    
	                                           I32 lMainAxisCenterPoint,	I32 lSlaveAxisCenterPoint, I32 lMainAxisFinishPoint, I32 lSlaveAxisFinishPoint,	        
	                                           U16 wInterpMode );

//--------------------------------------------------------------------------------------------------------------------------------
EXPORTS	I32 CALLBACK pcielm4_lines_move( U16 wBoardNo, U16 wAxes, U16 wAccDecMode,			
	                                      U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, U32 dwAcceleration, U32 dwDeceleration,		        
	                                      const I32 lRelativeDistance[PCIELM4_MAX_AXIS_NO], U16 wInterpMode	);

EXPORTS	I32 CALLBACK pcielm4_abs_lines_move( U16 wBoardNo, U16 wAxes, U16 wAccDecMode,			
	                                          U32 dwStartSpeed, U32 dwDriveSpeed, U32 dwEndSpeed, U32 dwAcceleration, U32 dwDeceleration,		        
	                                          const I32 lAbsolutePosition[PCIELM4_MAX_AXIS_NO], U16 wInterpMode	);

EXPORTS	I32 CALLBACK pcielm4_drv_hold(U16 wBoardNo, U16 wBitMultiAxes );
EXPORTS	I32 CALLBACK pcielm4_drv_start(U16 wBoardNo, U16 wBitMultiAxes );
EXPORTS	I32 CALLBACK pcielm4_stop_move(U16 wBoardNo, U8 bSingleAxis, U8 bStopMode );
EXPORTS	I32 CALLBACK pcielm4_reset( U16 wBoardNo );

EXPORTS	I32 CALLBACK pcielm4_set_latch_cfg(U16 wBoardNo, U8 bSingleAxis, U8 bLatchEnable, U8 bLatchLogic );
EXPORTS	I32 CALLBACK pcielm4_get_latch(U16 wBoardNo, U8 bSingleAxis, I32* plLatchData );
EXPORTS	I32 CALLBACK pcielm4_get_error_status(U16 wBoardNo, U8 bSingleAxis, U16* pwErrorStatus );
EXPORTS	I32 CALLBACK pcielm4_get_mdi_status(U16 wBoardNo, U8 bSingleAxis, U16* pwDIStatus );
EXPORTS	I32 CALLBACK pcielm4_get_servo_on_status(U16 wBoardNo, U8 bSingleAxis, U8* pbServoOn );
EXPORTS	I32 CALLBACK pcielm4_get_servo_erc_status(U16 wBoardNo, U8 bSingleAxis, U8* pbServoErc );
EXPORTS	I32 CALLBACK pcielm4_get_servo_almrst_status(U16 wBoardNo, U8 bSingleAxis, U8* pbServoAlarm );

//--------------------------------------------------------------------------------------------------------------------------------
//    APIs defined for direct DDA control:
//--------------------------------------------------------------------------------------------------------------------------------
EXPORTS	I32 CALLBACK pcielm4_set_servo_period( U16 wBoardNo, U16 wDDALength );
EXPORTS	I32 CALLBACK pcielm4_set_ctrl_mode( U16 wBoardNo, U8 bSingleAxis, U16 wCtlMode );
EXPORTS	I32 CALLBACK pcielm4_set_target_real_pos( U16 wBoardNo, U16 wMultiAxes, const I32 lDdaValue[PCIELM4_MAX_AXIS_NO] );
EXPORTS	I32 CALLBACK pcielm4_set_current_real_pos( U16 wBoardNo, U16 wMultiAxes, const I32 lAbsDdaVal[PCIELM4_MAX_AXIS_NO]);
EXPORTS	I32 CALLBACK pcielm4_stop_real_axes( U16 wBoardNo, U16 wMultiAxes	);
EXPORTS	I32 CALLBACK pcielm4_set_ctrl_period_mult(U16 wBoardNo, U16 wMultiplier );

//--------------------------------------------------------------------------------------------------------------------------------
//    Buffer mode commands
//--------------------------------------------------------------------------------------------------------------------------------
EXPORTS	I32 CALLBACK pcielm4_buffer_initial( U16 wBoardNo, U8 bSingleAxis, U16 wBufferType, U32 dwBufferU32Size);
EXPORTS	I32 CALLBACK pcielm4_buffer_reinit( U16 wBoardNo, U8 bSingleAxis, U16 wBufferType );
EXPORTS	I32 CALLBACK pcielm4_buffer_release( U16 wBoardNo, U8 bSingleAxis, U16 wBufferType );
EXPORTS	I32 CALLBACK pcielm4_buffer_continue( U16 wBoardNo, U8 bSingleAxis, U16 wBufferType, U16 wU32Qty, U8 bFinish, U32 * pdwBufData );
EXPORTS	I32 CALLBACK pcielm4_buffer_start( U16 wBoardNo, U8 bSingleAxis, U16 wBufferType, U16 wStep, U8 bLoop, U8 bRestart, U8 blnHold);
EXPORTS	I32 CALLBACK pcielm4_buffer_pause( U16 wBoardNo, U8 bSingleAxis, U16 wBufferType);

#endif // _pcielm4_LIB_H_