
// PCIELM4_DdaCtrlDlg.cpp : implementation file
//

#include "stdafx.h"
#include "PCIELM4_DdaCtrl.h"
#include "PCIELM4_DdaCtrlDlg.h"

#include "PCIELM4_MC.h"
#include "ErrorStr.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#define BOARD_NO        0
#define MAX_AXIS_NUM	2

#define AXIS_DO_SVON	0
#define AXIS_DO_RALM	1

#define MOVE_TYPE_RELATIVE	0
#define MOVE_TYPE_ABSOLUTE	1
#define MOVE_TYPE_VELOCITY	2

#define SPEED_TYPE_T_CURVE	0
#define SPEED_TYPE_S_CURVE	1

#define WM_USER_DDA_THREAD_FINISHED   (WM_APP + 0x100)
#define WM_USER_DDA_THREAD_ERROR      (WM_APP + 0x101)


#define _CHK(PCIELM4_API)\
	lRet = PCIELM4_API;\
	if( lRet != PCIELM4_SUCCESS)\
	{\
	CStringA strApi;\
	CStringW strRet;\
	strApi.Format("" #PCIELM4_API "\n");\
	strRet = GetErrorString(lRet, strApi, m_bBoardNo, AxisNo);\
	c_listLog.InsertString(0, strRet);\
	c_listLog.SetCurSel(0);\
	}


struct THREAD_DDA_PARA
{
	HWND hWnd;
	BYTE bBoardNo;
	long lDdaValue[2];
	DWORD dwCycleTime;
	DWORD dwCycleQty;
};

//---------------------------------------------------------
// DDA thread execution
//---------------------------------------------------------
UINT DDA_Thread(LPVOID pParam)
{
	HWND hWnd;
	BYTE bBoardNo;
	long lDdaValue[2];
	DWORD dwCycleTime;
	DWORD dwCycleQty;
	DWORD i;

	THREAD_DDA_PARA *pThreadPara = (THREAD_DDA_PARA*) pParam;
	hWnd = pThreadPara->hWnd;
	bBoardNo = pThreadPara->bBoardNo;
	lDdaValue[0] = pThreadPara->lDdaValue[0];
	lDdaValue[1] = pThreadPara->lDdaValue[1];
	dwCycleTime = pThreadPara->dwCycleTime;
	dwCycleQty = pThreadPara->dwCycleQty;
	delete pThreadPara;
	

	//----------------------------------------------
	//Configure and Initialize the DDA
	//----------------------------------------------
	BYTE AxisNo; //= PCIELM4_AXIS_0| PCIELM4_AXIS_1;
	long lLogicPosCount[MAX_AXIS_QTY];
    long lAbsDdaVal[MAX_AXIS_QTY] = {0};

    //Set servo period to 1 millisecond:
    pcielm4_set_servo_period(bBoardNo, 25000 );
    pcielm4_set_ctrl_period_mult(bBoardNo, 4 );

    for(i=0; i<MAX_AXIS_QTY; i++)
    {
        AxisNo = (1<<i);
        pcielm4_set_ctrl_mode( bBoardNo, AxisNo, CTL_MODE_VIRTUAL_PULSE );
        pcielm4_get_cmdcounter(bBoardNo, AxisNo, &lLogicPosCount[i] );

        //For testing connected the DDA directly to encoder channel
		//Set the encoder to 12MHz by disabling the filter:
		pcielm4_set_axis_di_filter( bBoardNo, AxisNo, ENABLE, AXIS_DI_EA_CH, FILTER_DELAY_DISABLE  );
		pcielm4_set_axis_di_filter( bBoardNo, AxisNo, ENABLE, AXIS_DI_EB_CH, FILTER_DELAY_DISABLE  );

    }
    
    AxisNo = AXIS_0| AXIS_1;
    pcielm4_set_current_real_pos( bBoardNo,  (WORD)AxisNo, &lLogicPosCount[0]);


	//----------------------------------------------
	//Each cycle output the  incremental DDA value:
	//----------------------------------------------
	for(i=0; i<dwCycleQty; i++)
	{
		lLogicPosCount[0] += lDdaValue[0];
		lLogicPosCount[1] += lDdaValue[1];

		//Trigger the DDA to output the difference between the current real position and the target position: 
		pcielm4_set_target_real_pos( bBoardNo, AxisNo, &lLogicPosCount[0] );

		::Sleep(dwCycleTime);
	}


	//----------------------------------------------
	//Set the PCIELM4 back to the default mode:
	//----------------------------------------------
	//Set servo period to 0.5 millisecond:
    pcielm4_set_servo_period(bBoardNo, 12500 );  // default value

    for(i=0; i<MAX_AXIS_QTY; i++)
    {
        AxisNo = (1<<i);
        pcielm4_set_ctrl_mode( bBoardNo, AxisNo, CTL_MODE_PULSE );
        //pcielm4_set_cmdcounter(bBoardNo, AxisNo, 0 );
        //pcielm4_set_enccounter(bBoardNo, AxisNo, 0 );
    
		 //For testing connected the DDA directly to encoder channel
		//Set the encoder to 12MHz by disabling the filter:
		pcielm4_set_axis_di_filter( bBoardNo, AxisNo, ENABLE, AXIS_DI_EA_CH, FILTER_DELAY_DISABLE );
		pcielm4_set_axis_di_filter( bBoardNo, AxisNo, ENABLE, AXIS_DI_EB_CH, FILTER_DELAY_DISABLE );

	}
	::PostMessageW(hWnd, WM_USER_DDA_THREAD_FINISHED, 0, 0);
	   
	return 0;
}




// CPCIELM4_DdaCtrlDlg dialog
CPCIELM4_DdaCtrlDlg::CPCIELM4_DdaCtrlDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPCIELM4_DdaCtrlDlg::IDD, pParent)
	,m_bBoardNo(BOARD_NO)
	,m_flgCardInitialized(FALSE)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CPCIELM4_DdaCtrlDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_lstLog, c_listLog);
	DDX_Control(pDX, IDC_EDIT_BOARD_NO, c_BoardNo);
	DDX_Control(pDX, IDC_btnConnect, c_btnConnect);
	DDX_Control(pDX, IDC_btnDisconnect, c_btnDisconnect);
	DDX_Control(pDX, IDC_BTN_START_DDA_CTRL, c_btnStartDdaOutput);
	DDX_Control(pDX, IDC_btnSvon0, c_btnServoOnAxis0);
	DDX_Control(pDX, IDC_btnSvon1, c_btnServoOnAxis1);
	DDX_Control(pDX, IDC_btnRALM0, c_btnServoResetAlrmAxis0);
	DDX_Control(pDX, IDC_btnRALM1, c_btnServoResetAlrmAxis1);
	DDX_Control(pDX, IDC_btnResetCnt0, c_btnServoResetAxis0);
	DDX_Control(pDX, IDC_btnResetCnt1, c_btnServoResetAxis1);
	DDX_Control(pDX, IDC_btnResetCntAll, c_btnServoResetAll);
}

BEGIN_MESSAGE_MAP(CPCIELM4_DdaCtrlDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_WM_TIMER()
	ON_MESSAGE(WM_USER_DDA_THREAD_FINISHED, &CPCIELM4_DdaCtrlDlg::OnDdaThreadFinished)
	ON_CONTROL_RANGE(BN_CLICKED, IDC_btnResetCnt0, IDC_btnResetCnt1, OnbtnResetCnt)
	ON_CONTROL_RANGE(BN_CLICKED, IDC_btnSvon0, IDC_btnSvon1, OnbtnSvon)
	ON_CONTROL_RANGE(BN_CLICKED, IDC_btnRALM0, IDC_btnRALM1, OnbtnRALM)
	ON_BN_CLICKED(IDC_btnResetCntAll, &CPCIELM4_DdaCtrlDlg::OnBnClickedbtnresetcntall)
	ON_BN_CLICKED(IDC_btnClearLog, &CPCIELM4_DdaCtrlDlg::OnBnClickedbtnclearlog)
	ON_BN_CLICKED(IDC_btnConnect, &CPCIELM4_DdaCtrlDlg::OnBnClickedbtnconnect)
	ON_BN_CLICKED(IDC_btnDisconnect, &CPCIELM4_DdaCtrlDlg::OnBnClickedbtndisconnect)
	ON_BN_CLICKED(IDC_BTN_START_DDA_CTRL, &CPCIELM4_DdaCtrlDlg::OnBnClickedBtn_StartDdaCtrl)
END_MESSAGE_MAP()


// CPCIELM4_DdaCtrlDlg message handlers

BOOL CPCIELM4_DdaCtrlDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

		//----------------------------------------
	//Connection:
	//----------------------------------------
	c_BoardNo.SetWindowTextW(L"0");
	ButtonEnable(FALSE);


	//----------------------------------------
	//DDA Control:
	//----------------------------------------
	SetDlgItemInt(IDC_EDIT_CYCLE_TIME, 100);
	SetDlgItemInt(IDC_EDIT_CYCLE_QTY, 100);
	SetDlgItemInt(IDC_EDIT_INCR_DDA_VALUE_X, 1000);
	SetDlgItemInt(IDC_EDIT_INCR_DDA_VALUE_Y, 1000);

	SetTimer(1, 10, NULL);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CPCIELM4_DdaCtrlDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CPCIELM4_DdaCtrlDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


void CPCIELM4_DdaCtrlDlg::OnTimer(UINT_PTR nIDEvent)
{
	long lRet = 0;
	int i = 0;
	CString str;
	BYTE AxisNo = 0;
	long LogicPos = 0, EncPos = 0, Speed = 0 ;
	BYTE Done = 0;
	WORD StopStatus = 0;
	WORD IOStatus = 0/*, DIStatus = 0*/;
	static BOOL flgDisCon = FALSE; 

	if ( m_flgCardInitialized == TRUE )
	{
		for (i = 0; i < MAX_AXIS_NUM; i++)
		{
			AxisNo = 0x01 << i;

			_CHK(pcielm4_get_cmdcounter(m_bBoardNo, AxisNo, &LogicPos) );
			SetDlgItemInt(IDC_edtCmdCnt0 + i, LogicPos);

			_CHK(pcielm4_get_enccounter(m_bBoardNo, AxisNo, &EncPos) );
			SetDlgItemInt(IDC_edtEncCnt0 + i, EncPos);

			_CHK(pcielm4_get_speed(m_bBoardNo, AxisNo, &Speed) );
			SetDlgItemInt(IDC_edtCurSpeed0 + i, Speed);

			_CHK(pcielm4_get_motion_done( m_bBoardNo, AxisNo, &Done, &StopStatus )  );
			str.Format(L"%d_0x%04x", Done, StopStatus);
			SetDlgItemText(IDC_edtMotionSts0 + i, str);

			_CHK(pcielm4_get_mdi_status( m_bBoardNo, AxisNo, &IOStatus )  );
			str.Format(L"0x%04x", IOStatus);
			SetDlgItemText(IDC_edtIOSts0 + i, str);
		}

		flgDisCon = FALSE;
	}
	else
	{
		if (flgDisCon == FALSE)
		{
			flgDisCon = TRUE;
			for (i = 0; i < MAX_AXIS_NUM; i++)
			{
				SetDlgItemText(IDC_edtCmdCnt0 + i, L"NULL");
				SetDlgItemText(IDC_edtEncCnt0 + i, L"NULL");
				SetDlgItemText(IDC_edtCurSpeed0 + i, L"NULL");
				SetDlgItemText(IDC_edtMotionSts0 + i, L"NULL");
				SetDlgItemText(IDC_edtIOSts0 + i, L"NULL");
			}
		}
	}

	CDialog::OnTimer(nIDEvent);
}

void CPCIELM4_DdaCtrlDlg::OnBnClickedbtnconnect()
{
	long lRet = 0;
	BYTE AxisNo = 0;
	int i = 0;
	CStringW strTemp;
	WORD Ver = 0, FWVer = 0, DSPVer = 0, PCBVer = 0, PLDVer = 0, FPGAVer = 0;
	
	m_flgCardInitialized = FALSE;

	c_BoardNo.GetWindowTextW(strTemp);
	m_bBoardNo = (BYTE) _wtol(strTemp);

	c_btnConnect.EnableWindow(FALSE);


	//-------------------------------
	//STEP 1: Open Driver
	//-------------------------------
	lRet = pcielm4_open_driver();
	if(lRet != PCIELM4_SUCCESS)
	{
		AfxMessageBox(L"Open PCIELM4 driver failed!");
		c_btnConnect.EnableWindow(TRUE);
		c_btnDisconnect.EnableWindow(FALSE);
		return;
	}

	//-------------------------------
	//STEP 2: Check PCIELM4 is in PCIe Slot
	//-------------------------------
	lRet =  pcielm4_check_card_in_slot( m_bBoardNo );
	if(lRet != PCIELM4_SUCCESS)
	{
		AfxMessageBox(L"PCIELM4 board %d was not detected!", m_bBoardNo);
		c_btnConnect.EnableWindow(TRUE);
		c_btnDisconnect.EnableWindow(FALSE);
		return;
	}

	//-------------------------------
	//STEP 3: Intialize PCIELM4 card
	//-------------------------------
	pcielm4_registration( m_bBoardNo );

	m_flgCardInitialized = TRUE;
	
	//WritePrivateProfileString("TestParam", "IP", strTemp, ".\\TestParam.ini");

	_CHK(pcielm4_get_dsp_firmware_version( m_bBoardNo, &DSPVer) );
	_CHK(pcielm4_get_card_version( m_bBoardNo, &PCBVer, &PLDVer) );
	_CHK(pcielm4_get_fpga_version( m_bBoardNo, &FPGAVer) );
	strTemp.Format(L"Firmware Ver.: 0x%04x, PCB Ver.: 0x%04x", DSPVer, PCBVer);
	SetDlgItemText(IDC_edtCardInfo, strTemp);

	IniSetting();
	ButtonEnable(TRUE);
}

void CPCIELM4_DdaCtrlDlg::OnBnClickedbtndisconnect()
{
	m_flgCardInitialized = FALSE;
	pcielm4_close_driver();

	ButtonEnable(FALSE);
}

void CPCIELM4_DdaCtrlDlg::OnbtnResetCnt(UINT nID)
{
	BYTE Axis = nID - IDC_btnResetCnt0;

	ResetAxisCnt(Axis);
}

void CPCIELM4_DdaCtrlDlg::OnBnClickedbtnresetcntall()
{
	BYTE Axis = 0;

	for (Axis = 0; Axis < MAX_AXIS_NUM; Axis++)
	{ResetAxisCnt(Axis);}
}

void CPCIELM4_DdaCtrlDlg::OnbtnSvon(UINT nID)
{
	BYTE Axis = nID - IDC_btnSvon0;
	AxisDO(Axis, AXIS_DO_SVON);
}

void CPCIELM4_DdaCtrlDlg::OnbtnRALM(UINT nID)
{
	BYTE Axis = nID - IDC_btnRALM0;
	AxisDO(Axis, AXIS_DO_RALM);
}

void CPCIELM4_DdaCtrlDlg::OnBnClickedbtnclearlog()
{
	c_listLog.ResetContent();
}


int CPCIELM4_DdaCtrlDlg::ResetAxisCnt(BYTE Axis)
{
	BYTE AxisNo = 0x01 << Axis;
	long LogicPos = 0;
	long EncPos = 0;
	long lRet = 0;

	_CHK(pcielm4_set_cmdcounter(m_bBoardNo, AxisNo, LogicPos) );
	_CHK(pcielm4_set_enccounter(m_bBoardNo, AxisNo, EncPos) );

	return 0;
}

int CPCIELM4_DdaCtrlDlg::AxisDO(BYTE Axis, BYTE Ch)
{
	BYTE AxisNo = 0x01 << Axis;
	long lRet = 0;
	BYTE OnOff = FALSE;

	if (Ch == AXIS_DO_SVON)
	{
		_CHK(pcielm4_get_servo_on_status(m_bBoardNo, AxisNo, &OnOff ) );

		if (OnOff)
		{_CHK(pcielm4_set_servo_on( m_bBoardNo, AxisNo, FALSE ) );}
		else
		{_CHK(pcielm4_set_servo_on( m_bBoardNo, AxisNo, TRUE ) );}

	}
	else if (Ch == AXIS_DO_RALM)
	{        
		_CHK(pcielm4_get_servo_almrst_status(m_bBoardNo, AxisNo, &OnOff ) );
		if (OnOff)
		{_CHK(pcielm4_set_alarm_reset( m_bBoardNo, AxisNo, FALSE ) );}
		else
		{_CHK(pcielm4_set_alarm_reset( m_bBoardNo, AxisNo, TRUE ) );}		
	}
	return 0;
}

int CPCIELM4_DdaCtrlDlg::IniSetting(void)
{
	int i = 0;
	long lRet = 0;
	BYTE AxisNo = 0;

	BYTE HomeSteps = AUTO_HOME_STEP1_ENABLE | AUTO_HOME_STEP2_ENABLE | AUTO_HOME_STEP3_ENABLE | AUTO_HOME_STEP4_ENABLE; 

	_CHK(pcielm4_set_conti_interp_cfg(m_bBoardNo, FALSE, 0x03 ) );

	for (i = 0; i < MAX_AXIS_NUM; i++)
	{
		AxisNo = 0x0001 << i;	

		_CHK(pcielm4_set_pls_cfg(m_bBoardNo, AxisNo, PULSE_MODE_AB_DIVID_4, FALSE, FALSE ) );
		_CHK(pcielm4_set_enc_cfg(m_bBoardNo, AxisNo, ENCODER_MODE_AB_DIVID_4 ) );

		_CHK(pcielm4_set_limit(m_bBoardNo, AxisNo, FALSE, STOP_SUDDEN) );
		//_CHK(pcielm4_set_softlimit( m_bBoardNo, AxisNo, STOP_SUDDEN, REF_CMD_CNT, 0x7FFFFFFF, -0x7FFFFFFF) );
		_CHK(pcielm4_set_softlimit_disable(m_bBoardNo, AxisNo) );
		_CHK(pcielm4_set_home_cfg(m_bBoardNo, AxisNo, FALSE, FALSE, FALSE, HomeSteps, 0) );
		_CHK(pcielm4_set_alarm( m_bBoardNo, AxisNo, TRUE, FALSE ) );
		_CHK(pcielm4_set_inp( m_bBoardNo, AxisNo, FALSE, FALSE ) );
		_CHK(pcielm4_set_ready( m_bBoardNo, AxisNo, FALSE, FALSE ) );
		_CHK(pcielm4_set_latch_cfg( m_bBoardNo, AxisNo, FALSE, FALSE ) );
		_CHK(pcielm4_disable_vring_counter( m_bBoardNo, AxisNo) );
	}

	return 0;
}

void CPCIELM4_DdaCtrlDlg::OnBnClickedBtn_StartDdaCtrl()
{
	long lRet = 0;
	BYTE AxisNo = 0;

	c_btnStartDdaOutput.EnableWindow(FALSE);

	CWinThread *pThread;
	THREAD_DDA_PARA *pThreadPara = new THREAD_DDA_PARA;
	pThreadPara->hWnd = m_hWnd;
	pThreadPara->bBoardNo = m_bBoardNo;
	pThreadPara->lDdaValue[0] = GetDlgItemInt(IDC_EDIT_INCR_DDA_VALUE_X);
	pThreadPara->lDdaValue[1] = GetDlgItemInt(IDC_EDIT_INCR_DDA_VALUE_Y);
	pThreadPara->dwCycleTime = GetDlgItemInt(IDC_EDIT_CYCLE_TIME);
	pThreadPara->dwCycleQty = GetDlgItemInt(IDC_EDIT_CYCLE_QTY);
	pThread = AfxBeginThread(DDA_Thread, pThreadPara);
	if(pThread == NULL)
	{
		AfxMessageBox(L"DDA execution thread failed!");
		c_btnStartDdaOutput.EnableWindow(TRUE);
	}
}

LRESULT CPCIELM4_DdaCtrlDlg::OnDdaThreadFinished(WPARAM wParam, LPARAM lParam)
{
	c_btnStartDdaOutput.EnableWindow(TRUE);
	return 0;
}

void CPCIELM4_DdaCtrlDlg::ButtonEnable(BOOL flgTrueFalse)
{
	c_btnConnect.EnableWindow( (flgTrueFalse==TRUE)?FALSE:TRUE);
	c_btnDisconnect.EnableWindow(flgTrueFalse);
	c_btnStartDdaOutput.EnableWindow(flgTrueFalse);
	c_btnServoOnAxis0.EnableWindow(flgTrueFalse);
	c_btnServoOnAxis1.EnableWindow(flgTrueFalse);
	c_btnServoResetAlrmAxis0.EnableWindow(flgTrueFalse);
	c_btnServoResetAlrmAxis1.EnableWindow(flgTrueFalse);
	c_btnServoResetAxis0.EnableWindow(flgTrueFalse);
	c_btnServoResetAxis1.EnableWindow(flgTrueFalse);
	c_btnServoResetAll.EnableWindow(flgTrueFalse);
}
