
// PCIELM4_HomingDlg.cpp : implementation file
//

#include "stdafx.h"
#include "PCIELM4_Homing.h"
#include "PCIELM4_HomingDlg.h"


#include "ErrorStr.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif



#define BOARD_NO        0
#define MAX_AXIS_NUM	PCIELM4_MAX_AXIS_NO

#define AXIS_DO_SVON	0
#define AXIS_DO_RALM	1

#define MOVE_TYPE_RELATIVE	0
#define MOVE_TYPE_ABSOLUTE	1
#define MOVE_TYPE_VELOCITY	2

#define SPEED_TYPE_T_CURVE	0
#define SPEED_TYPE_S_CURVE	1


#define _CHK(PCIELM4_API)\
	lRet = PCIELM4_API;\
	if( lRet != PCIELM4_SUCCESS)\
	{\
	CStringA strApi;\
	CStringW strRet;\
	strApi.Format("" #PCIELM4_API "\n");\
	strRet = GetErrorString(lRet, strApi, m_bBoardNo, m_bAxisNo);\
	c_listLog.InsertString(0, strRet);\
	c_listLog.SetCurSel(0);\
	}



// CPCIELM4_HomingDlg dialog
CPCIELM4_HomingDlg::CPCIELM4_HomingDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPCIELM4_HomingDlg::IDD, pParent)
	,m_bBoardNo(BOARD_NO)
	,m_flgCardInitialized(FALSE)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CPCIELM4_HomingDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_lstLog, c_listLog);
	DDX_Control(pDX, IDC_EDIT_BOARD_NO, c_BoardNo);
	DDX_Control(pDX, IDC_btnConnect, c_btnConnect);
	DDX_Control(pDX, IDC_btnDisconnect, c_btnDisconnect);
	DDX_Control(pDX, IDC_btnSvon0, c_btnServoOnAxis);
	DDX_Control(pDX, IDC_btnRALM0, c_btnServoResetAlrmAxis);
	DDX_Control(pDX, IDC_btnResetCnt0, c_btnServoResetAxis);
	DDX_Control(pDX, IDC_btnMove0, c_btnMoveAxis);
	DDX_Control(pDX, IDC_btnStop0, c_btnStopAxis);
	DDX_Control(pDX, IDC_cmbAxisNo, c_cboAxisNo);
	DDX_Control(pDX, IDC_btnSetHome, c_btnHomeConfig);
	DDX_Control(pDX, IDC_btnHomeMove, c_btnStartHome);
	DDX_Control(pDX, IDC_btnStopHome, c_btnHomeStop);
}

BEGIN_MESSAGE_MAP(CPCIELM4_HomingDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_btnClearLog, &CPCIELM4_HomingDlg::OnBnClickedbtnclearlog)
	ON_BN_CLICKED(IDC_btnConnect, &CPCIELM4_HomingDlg::OnBnClickedbtnconnect)
	ON_BN_CLICKED(IDC_btnDisconnect, &CPCIELM4_HomingDlg::OnBnClickedbtndisconnect)

	ON_BN_CLICKED(IDC_btnStop0, &CPCIELM4_HomingDlg::OnBnClickedbtn_Stop)
	ON_BN_CLICKED(IDC_btnMove0, &CPCIELM4_HomingDlg::OnBnClickedbtn_Move)
	ON_BN_CLICKED(IDC_btnSvon0, &CPCIELM4_HomingDlg::OnBnClickedbtn_ServoON)
	ON_BN_CLICKED(IDC_btnRALM0, &CPCIELM4_HomingDlg::OnBnClickedbtn_ResetAlarm)
	ON_BN_CLICKED(IDC_btnResetCnt0, &CPCIELM4_HomingDlg::OnBnClickedbtn_ResetCnt)
	ON_BN_CLICKED(IDC_btnSetHome, &CPCIELM4_HomingDlg::OnBnClickedbtn_Sethome)
	ON_BN_CLICKED(IDC_btnHomeMove, &CPCIELM4_HomingDlg::OnBnClickedbtn_HomeMove)
	ON_BN_CLICKED(IDC_btnStopHome, &CPCIELM4_HomingDlg::OnBnClickedbtn_StopHome)
	ON_CBN_SELCHANGE(IDC_cmbAxisNo, &CPCIELM4_HomingDlg::OnCbnSelchange_AxisNo)
END_MESSAGE_MAP()

// CPCIELM4_HomingDlg message handlers

BOOL CPCIELM4_HomingDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	I32 i = 0;

	//-------------------------
	//Initialize the textboxes:
	//-------------------------
	c_BoardNo.SetWindowTextW(L"0");
	c_btnConnect.EnableWindow(TRUE);
	ButtonEnable(FALSE);
	
	SetDlgItemInt(IDC_edtSpeed, 10000);
	SetDlgItemInt(IDC_edtAcc, 20000);
	SetDlgItemInt(IDC_edtDist, 20000);

	c_cboAxisNo.SetCurSel(0);
	m_bAxisNo = 0x01;


	//-------------------------
	//Load Icon:
	//-------------------------
	//m_hBmpErr = LoadBitmap(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_bmpRed));
	m_hBmpOn = LoadBitmap(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_LED_RED));
	m_hBmpOff = LoadBitmap(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_LED_BLACK));


	//-------------------------
	//Initial home setting:
	//-------------------------
	for (i = 0; i < MAX_AXIS_NUM; i++)
	{
		m_stHome[i].HomeLogic = TRUE;
		m_stHome[i].SLDLogic = TRUE;
		m_stHome[i].EZLogic = TRUE;
		m_stHome[i].HomeSteps = AUTO_HOME_STEP1_ENABLE | AUTO_HOME_STEP2_ENABLE | AUTO_HOME_STEP3_ENABLE | AUTO_HOME_STEP4_ENABLE;
		m_stHome[i].Step4Offset = 0;

		m_stHome[i].StartSpeed = 1000;
		m_stHome[i].Acceleration = 80000;
		m_stHome[i].Deceleration = 80000;
		m_stHome[i].HighSpeed = 10000;
		m_stHome[i].LowSpeed = 2000;
		m_stHome[i].HomingDirection = 0;

	}
	ReflashHomeData(0);

	SetTimer(1, 10, NULL);
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CPCIELM4_HomingDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CPCIELM4_HomingDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}



void CPCIELM4_HomingDlg::OnTimer(UINT_PTR nIDEvent)
{
	I32 lRet = 0;
	CString str;
	I32 LogicPos = 0, EncPos = 0, Speed = 0 ;
	BYTE Done = 0;
	WORD StopStatus = 0;
	WORD IOStatus = 0/*, DIStatus = 0*/;
	static BOOL flgDisCon = FALSE; 

	if ( m_flgCardInitialized == TRUE )
	{
		_CHK(pcielm4_get_cmdcounter(m_bBoardNo, m_bAxisNo, &LogicPos) );
		SetDlgItemInt(IDC_edtCmdCnt, LogicPos);

		_CHK(pcielm4_get_enccounter(m_bBoardNo, m_bAxisNo, &EncPos) );
		SetDlgItemInt(IDC_edtEncCnt, EncPos);

		_CHK(pcielm4_get_speed(m_bBoardNo, m_bAxisNo, &Speed) );
		SetDlgItemInt(IDC_edtCurSpeed, Speed);

		_CHK(pcielm4_get_motion_done( m_bBoardNo, m_bAxisNo, &Done, &StopStatus )  );
		str.Format(L"%d_0x%04x", Done, StopStatus);
		SetDlgItemText(IDC_edtMotionSts, str);

		_CHK(pcielm4_get_mdi_status( m_bBoardNo, m_bAxisNo, &IOStatus )  );
		str.Format(L"0x%04x", IOStatus);
		SetDlgItemText(IDC_edtIOSts, str);

		if (IOStatus & DI_STATUS_ACTIVE_EMG)
		{ ((CStatic *)GetDlgItem(IDC_stcEMG))->SetBitmap(m_hBmpOn);}
		else
		{ ((CStatic *)GetDlgItem(IDC_stcEMG))->SetBitmap(m_hBmpOff);}

		if (IOStatus & DI_STATUS_ACTIVE_HOME)
		{ ((CStatic *)GetDlgItem(IDC_stcORG))->SetBitmap(m_hBmpOn);}
		else
		{ ((CStatic *)GetDlgItem(IDC_stcORG))->SetBitmap(m_hBmpOff);}

		if (IOStatus & DI_STATUS_ACTIVE_NEARHOME)
		{ ((CStatic *)GetDlgItem(IDC_stcSLD))->SetBitmap(m_hBmpOn);}
		else
		{ ((CStatic *)GetDlgItem(IDC_stcSLD))->SetBitmap(m_hBmpOff);}

		if (IOStatus & DI_STATUS_ACTIVE_INDEX)
		{ ((CStatic *)GetDlgItem(IDC_stcEZ))->SetBitmap(m_hBmpOn);}
		else
		{ ((CStatic *)GetDlgItem(IDC_stcEZ))->SetBitmap(m_hBmpOff);}

		if (IOStatus & DI_STATUS_ACTIVE_LMTP)
		{ ((CStatic *)GetDlgItem(IDC_stcPEL))->SetBitmap(m_hBmpOn);}
		else
		{ ((CStatic *)GetDlgItem(IDC_stcPEL))->SetBitmap(m_hBmpOff);}

		if (IOStatus & DI_STATUS_ACTIVE_LMTM)
		{ ((CStatic *)GetDlgItem(IDC_stcMEL))->SetBitmap(m_hBmpOn);}
		else
		{ ((CStatic *)GetDlgItem(IDC_stcMEL))->SetBitmap(m_hBmpOff);}


		flgDisCon = FALSE;
	}
	else
	{
		if (flgDisCon == FALSE)
		{
			flgDisCon = TRUE;
			SetDlgItemText(IDC_edtCmdCnt, L"NULL");
			SetDlgItemText(IDC_edtEncCnt, L"NULL");
			SetDlgItemText(IDC_edtCurSpeed, L"NULL");
			SetDlgItemText(IDC_edtMotionSts, L"NULL");
			SetDlgItemText(IDC_edtIOSts, L"NULL");
		}
	}

	CDialog::OnTimer(nIDEvent);
}

void CPCIELM4_HomingDlg::OnBnClickedbtnconnect()
{
	I32 lRet = 0;
	int i = 0;
	CStringW strTemp;
	WORD Ver = 0, FWVer = 0, DSPVer = 0, PCBVer = 0, PLDVer = 0, FPGAVer = 0;
	
	m_flgCardInitialized = FALSE;

	c_BoardNo.GetWindowTextW(strTemp);
	m_bBoardNo = (BYTE) _wtol(strTemp);

	c_btnConnect.EnableWindow(FALSE);

	//-------------------------------
	//STEP 1: Open Driver
	//-------------------------------
	lRet = pcielm4_open_driver();
	if(lRet != PCIELM4_SUCCESS)
	{
		AfxMessageBox(L"Open PCIELM4 driver failed!");
		c_btnConnect.EnableWindow(TRUE);
		c_btnDisconnect.EnableWindow(FALSE);
		return;
	}

	//-------------------------------
	//STEP 2: Check PCIELM4 is in PCIe Slot
	//-------------------------------
	lRet =  pcielm4_check_card_in_slot( m_bBoardNo );
	if(lRet != PCIELM4_SUCCESS)
	{
		AfxMessageBox(L"PCIELM4 board %d was not detected!", m_bBoardNo);
		c_btnConnect.EnableWindow(TRUE);
		c_btnDisconnect.EnableWindow(FALSE);
		return;
	}

	//-------------------------------
	//STEP 3: Intialize PCIELM4 card
	//-------------------------------
	pcielm4_registration( m_bBoardNo );

	m_flgCardInitialized = TRUE;
	
	_CHK(pcielm4_get_dsp_firmware_version( m_bBoardNo, &DSPVer) );
	_CHK(pcielm4_get_card_version( m_bBoardNo, &PCBVer, &PLDVer) );
	_CHK(pcielm4_get_fpga_version( m_bBoardNo, &FPGAVer) );
	strTemp.Format(L"Firmware Ver.: 0x%04x, PCB Ver.: 0x%04x", DSPVer, PCBVer);
	SetDlgItemText(IDC_edtCardInfo, strTemp);

	IniSetting();
	ButtonEnable(TRUE);

}

void CPCIELM4_HomingDlg::OnBnClickedbtndisconnect()
{
	m_flgCardInitialized = FALSE;
	pcielm4_close_driver();

	ButtonEnable(FALSE);
}

void CPCIELM4_HomingDlg::OnBnClickedbtnclearlog()
{
	c_listLog.ResetContent();
}

int CPCIELM4_HomingDlg::IniSetting(void)
{
	int i = 0;
	I32 lRet = 0;
	U8 AxisNo;

	BYTE HomeSteps = AUTO_HOME_STEP1_ENABLE | AUTO_HOME_STEP2_ENABLE | AUTO_HOME_STEP3_ENABLE | AUTO_HOME_STEP4_ENABLE; 

	_CHK(pcielm4_set_conti_interp_cfg(m_bBoardNo, FALSE, 0x03 ) );

	for (i = 0; i < MAX_AXIS_NUM; i++)
	{
		AxisNo = 0x0001 << i;	

		_CHK(pcielm4_set_pls_cfg(m_bBoardNo, AxisNo, PULSE_MODE_AB_DIVID_4, FALSE, FALSE ) );
		_CHK(pcielm4_set_enc_cfg(m_bBoardNo, AxisNo, ENCODER_MODE_AB_DIVID_4 ) );

		_CHK(pcielm4_set_limit(m_bBoardNo, AxisNo, FALSE, STOP_SUDDEN) );
		//_CHK(pcielm4_set_softlimit( m_bBoardNo, AxisNo, STOP_SUDDEN, REF_CMD_CNT, 0x7FFFFFFF, -0x7FFFFFFF) );
		_CHK(pcielm4_set_softlimit_disable(m_bBoardNo, m_bAxisNo) );
		_CHK(pcielm4_set_home_cfg(m_bBoardNo, AxisNo, FALSE, FALSE, FALSE, HomeSteps, 0) );
		_CHK(pcielm4_set_alarm( m_bBoardNo, AxisNo, TRUE, FALSE ) );
		_CHK(pcielm4_set_inp( m_bBoardNo, AxisNo, FALSE, FALSE ) );
		_CHK(pcielm4_set_ready( m_bBoardNo, AxisNo, FALSE, FALSE ) );
		_CHK(pcielm4_set_latch_cfg( m_bBoardNo, AxisNo, FALSE, FALSE ) );
		_CHK(pcielm4_disable_vring_counter( m_bBoardNo, AxisNo) );
	}

	return 0;
}

void CPCIELM4_HomingDlg::ButtonEnable(BOOL flgTrueFalse)
{
	c_btnConnect.EnableWindow( (flgTrueFalse==TRUE)?FALSE:TRUE);
	c_btnDisconnect.EnableWindow(flgTrueFalse);
	c_btnServoOnAxis.EnableWindow(flgTrueFalse);
	c_btnServoResetAlrmAxis.EnableWindow(flgTrueFalse);
	c_btnServoResetAxis.EnableWindow(flgTrueFalse);
	c_btnMoveAxis.EnableWindow(flgTrueFalse);
	c_btnStopAxis.EnableWindow(flgTrueFalse);
	c_btnHomeConfig.EnableWindow(flgTrueFalse);
	c_btnStartHome.EnableWindow(flgTrueFalse);
	c_btnHomeStop.EnableWindow(flgTrueFalse);
}

void CPCIELM4_HomingDlg::OnBnClickedbtn_Stop()
{
	I32 lRet = 0;

	// StopMode = 1: slow down then stop, 2: sudden stop
	_CHK(pcielm4_stop_move( m_bBoardNo, m_bAxisNo, STOP_SLOWDOWN ) );

}

void CPCIELM4_HomingDlg::OnBnClickedbtn_Move()
{
	I32 lRet = 0;
	U32 Speed = GetDlgItemInt(IDC_edtSpeed);
	U32 Acc = GetDlgItemInt(IDC_edtAcc);
	U32 Dist = GetDlgItemInt(IDC_edtDist);

	_CHK(pcielm4_t_move( m_bBoardNo, m_bAxisNo, (U32)(Speed/10), (U32)Speed, (U32)(Speed/10),
		(U32)Acc, (U32)Acc, (I32)Dist ) );
}

void CPCIELM4_HomingDlg::OnBnClickedbtn_ServoON()
{
	I32 lRet = 0;
	U8 OnOff = FALSE;

	_CHK(pcielm4_get_servo_on_status(m_bBoardNo, m_bAxisNo, &OnOff ) );

	if (OnOff)
	{_CHK(pcielm4_set_servo_on( m_bBoardNo, m_bAxisNo, FALSE ) );}
	else
	{_CHK(pcielm4_set_servo_on( m_bBoardNo, m_bAxisNo, TRUE ) );}
}

void CPCIELM4_HomingDlg::OnBnClickedbtn_ResetAlarm()
{
	I32 lRet = 0;
	U8 OnOff = FALSE;

	_CHK(pcielm4_get_servo_almrst_status(m_bBoardNo, m_bAxisNo, &OnOff ) );
	if (OnOff)
	{_CHK(pcielm4_set_alarm_reset( m_bBoardNo, m_bAxisNo, FALSE ) );}
	else
	{_CHK(pcielm4_set_alarm_reset( m_bBoardNo, m_bAxisNo, TRUE ) );}		

}

void CPCIELM4_HomingDlg::OnBnClickedbtn_ResetCnt()
{
	I32 lRet = 0;
	I32 LogicPos = 0;
	I32 EncPos = 0;

	_CHK(pcielm4_set_cmdcounter(m_bBoardNo, m_bAxisNo, LogicPos) );
	_CHK(pcielm4_set_enccounter(m_bBoardNo, m_bAxisNo, EncPos) );

}

void CPCIELM4_HomingDlg::OnBnClickedbtn_Sethome()
{
	int i = 0;
	I32 lRet = 0;
	U8 HomeLogic = 0, SLDLogic = 0, EZLogic = 0, HomeSteps = 0;
	I32 Step4Offset = 0;

	if (((CButton *)GetDlgItem(IDC_rdoHLogicH))->GetCheck())
	{HomeLogic = TRUE;}
	else
	{HomeLogic = FALSE;}

	if (((CButton *)GetDlgItem(IDC_rdoNHLogicH))->GetCheck())
	{SLDLogic = TRUE;}
	else
	{SLDLogic = FALSE;}

	if (((CButton *)GetDlgItem(IDC_rdoEZLogicH))->GetCheck())
	{EZLogic = TRUE;}
	else
	{EZLogic = FALSE;}

	if (((CButton *)GetDlgItem(IDC_chkHomeStep1))->GetCheck())
	{HomeSteps |= AUTO_HOME_STEP1_ENABLE;}

	if (((CButton *)GetDlgItem(IDC_chkHomeStep2))->GetCheck())
	{HomeSteps |= AUTO_HOME_STEP2_ENABLE;}

	if (((CButton *)GetDlgItem(IDC_chkHomeStep3))->GetCheck())
	{HomeSteps |= AUTO_HOME_STEP3_ENABLE;}

	if (((CButton *)GetDlgItem(IDC_chkHomeStep4))->GetCheck())
	{HomeSteps |= AUTO_HOME_STEP4_ENABLE;}

	Step4Offset = GetDlgItemInt(IDC_edtHomeOffset);

	_CHK(pcielm4_set_home_cfg(m_bBoardNo, m_bAxisNo, HomeLogic, SLDLogic, EZLogic, HomeSteps, Step4Offset));
	if (lRet == PCIELM4_SUCCESS)
	{
		for (i = 0; i < MAX_AXIS_NUM; i++)
		{
			if (m_bAxisNo & 0x0001 << i)
			{break;}
		}

		m_stHome[i].HomeLogic = HomeLogic;
		m_stHome[i].SLDLogic = SLDLogic;
		m_stHome[i].EZLogic = EZLogic;
		m_stHome[i].HomeSteps = HomeSteps;
		m_stHome[i].Step4Offset = Step4Offset;
	}

}

void CPCIELM4_HomingDlg::OnBnClickedbtn_HomeMove()
{
	I32 lRet = 0;
	int i = 0;

	U32 StartSpeed = (U32)GetDlgItemInt(IDC_edtHStrSpeed);
	U32 Acceleration = (U32)GetDlgItemInt(IDC_edtHAcc);
	U32 Deceleration = (U32)GetDlgItemInt(IDC_edtHDec);
	U32 HighSpeed = (U32)GetDlgItemInt(IDC_edtHNHSpeed);
	U32 LowSpeed = (U32)GetDlgItemInt(IDC_edtHHSpeed);
	U8 HomingDirection = 0;

	if (((CButton *)GetDlgItem(IDC_rdoHDirP))->GetCheck())
	{HomingDirection = 1;}
	else
	{HomingDirection = 0;}

	_CHK(pcielm4_home_start( m_bBoardNo, m_bAxisNo, StartSpeed, Acceleration, Deceleration,		
 		HighSpeed, LowSpeed, HomingDirection) );
	if (lRet == PCIELM4_SUCCESS)
	{
		for (i = 0; i < MAX_AXIS_NUM; i++)
		{
			if (m_bAxisNo & (0x0001 << i) )
			{break;}
		}

		m_stHome[i].StartSpeed = StartSpeed;
		m_stHome[i].Acceleration = Acceleration;
		m_stHome[i].Deceleration = Deceleration;
		m_stHome[i].HighSpeed = HighSpeed;
		m_stHome[i].LowSpeed = LowSpeed;
		m_stHome[i].HomingDirection = HomingDirection;
	}
	
}

void CPCIELM4_HomingDlg::OnBnClickedbtn_StopHome()
{
	I32 lRet = 0;
	_CHK(pcielm4_stop_move( m_bBoardNo, m_bAxisNo, STOP_SLOWDOWN ));

}

void CPCIELM4_HomingDlg::OnCbnSelchange_AxisNo()
{
	U8 SelAxis = c_cboAxisNo.GetCurSel();

	m_bAxisNo = 0x0001 << SelAxis;

	ReflashHomeData(SelAxis);
}

void CPCIELM4_HomingDlg::ReflashHomeData(BYTE AxisNo)
{

	if (m_stHome[AxisNo].HomeLogic)
	{
		((CButton *)GetDlgItem(IDC_rdoHLogicH))->SetCheck(TRUE);
		((CButton *)GetDlgItem(IDC_rdoHLogicL))->SetCheck(FALSE);
	}
	else
	{
		((CButton *)GetDlgItem(IDC_rdoHLogicH))->SetCheck(FALSE);
		((CButton *)GetDlgItem(IDC_rdoHLogicL))->SetCheck(TRUE);
	}

	if (m_stHome[AxisNo].SLDLogic)
	{
		((CButton *)GetDlgItem(IDC_rdoNHLogicH))->SetCheck(TRUE);
		((CButton *)GetDlgItem(IDC_rdoNHLogicL))->SetCheck(FALSE);
	}
	else
	{
		((CButton *)GetDlgItem(IDC_rdoNHLogicH))->SetCheck(FALSE);
		((CButton *)GetDlgItem(IDC_rdoNHLogicL))->SetCheck(TRUE);
	}

	if (m_stHome[AxisNo].EZLogic)
	{
		((CButton *)GetDlgItem(IDC_rdoEZLogicH))->SetCheck(TRUE);
		((CButton *)GetDlgItem(IDC_rdoEZLogicL))->SetCheck(FALSE);
	}
	else
	{
		((CButton *)GetDlgItem(IDC_rdoEZLogicH))->SetCheck(FALSE);
		((CButton *)GetDlgItem(IDC_rdoEZLogicL))->SetCheck(TRUE);
	}

	if (m_stHome[AxisNo].HomeSteps & AUTO_HOME_STEP1_ENABLE)
	{((CButton *)GetDlgItem(IDC_chkHomeStep1))->SetCheck(TRUE);}
	else
	{((CButton *)GetDlgItem(IDC_chkHomeStep1))->SetCheck(FALSE);}

	if (m_stHome[AxisNo].HomeSteps & AUTO_HOME_STEP2_ENABLE)
	{((CButton *)GetDlgItem(IDC_chkHomeStep2))->SetCheck(TRUE);}
	else
	{((CButton *)GetDlgItem(IDC_chkHomeStep2))->SetCheck(FALSE);}

	if (m_stHome[AxisNo].HomeSteps & AUTO_HOME_STEP3_ENABLE)
	{((CButton *)GetDlgItem(IDC_chkHomeStep3))->SetCheck(TRUE);}
	else
	{((CButton *)GetDlgItem(IDC_chkHomeStep3))->SetCheck(FALSE);}

	if (m_stHome[AxisNo].HomeSteps & AUTO_HOME_STEP4_ENABLE)
	{((CButton *)GetDlgItem(IDC_chkHomeStep4))->SetCheck(TRUE);}
	else
	{((CButton *)GetDlgItem(IDC_chkHomeStep4))->SetCheck(FALSE);}

	SetDlgItemInt(IDC_edtHomeOffset, m_stHome[AxisNo].Step4Offset);

	SetDlgItemInt(IDC_edtHStrSpeed, m_stHome[AxisNo].StartSpeed);
	SetDlgItemInt(IDC_edtHAcc, m_stHome[AxisNo].Acceleration);
	SetDlgItemInt(IDC_edtHDec, m_stHome[AxisNo].Deceleration);
	SetDlgItemInt(IDC_edtHNHSpeed, m_stHome[AxisNo].HighSpeed);
	SetDlgItemInt(IDC_edtHHSpeed, m_stHome[AxisNo].LowSpeed);

	if (m_stHome[AxisNo].HomingDirection)
	{
		((CButton *)GetDlgItem(IDC_rdoHDirP))->SetCheck(TRUE);
		((CButton *)GetDlgItem(IDC_rdoHDirN))->SetCheck(FALSE);
	}
	else
	{
		((CButton *)GetDlgItem(IDC_rdoHDirP))->SetCheck(FALSE);
		((CButton *)GetDlgItem(IDC_rdoHDirN))->SetCheck(TRUE);
	}
}
